@@ -330,6 +330,8 @@ class CompressDecompressFunctionTestCase(unittest.TestCase):
                           format=lzma.FORMAT_RAW, filters=FILTERS_RAW_4)
 
     def test_decompress_bad_input(self):
+        self.assertRaises(LZMAError, lzma.decompress,
+                          COMPRESSED_BOGUS);
         self.assertRaises(LZMAError, lzma.decompress,
                           COMPRESSED_RAW_1)
         self.assertRaises(LZMAError, lzma.decompress,
@@ -365,6 +367,15 @@ class CompressDecompressFunctionTestCase(unittest.TestCase):
         ddata = lzma.decompress(COMPRESSED_XZ + COMPRESSED_ALONE)
         self.assertEqual(ddata, INPUT * 2)
 
+    # Test robust handling of non-LZMA data following the compressed stream(s).
+
+    def test_decompress_trailing_junk(self):
+        ddata = lzma.decompress(COMPRESSED_XZ + COMPRESSED_BOGUS)
+        self.assertEqual(ddata, INPUT)
+
+    def test_decompress_multistream_trailing_junk(self):
+        ddata = lzma.decompress(COMPRESSED_XZ * 3 + COMPRESSED_BOGUS)
+        self.assertEqual(ddata, INPUT * 3)
 
 class TempFile:
     """Context manager - creates a file, and deletes it on __exit__."""
@@ -690,6 +701,14 @@ class FileTestCase(unittest.TestCase):
         finally:
             lzma._BUFFER_SIZE = saved_buffer_size
 
+    def test_read_trailing_junk(self):
+        with LZMAFile(BytesIO(COMPRESSED_XZ + COMPRESSED_BOGUS)) as f:
+            self.assertEqual(f.read(), INPUT)
+
+    def test_read_multistream_trailing_junk(self):
+        with LZMAFile(BytesIO(COMPRESSED_XZ * 5 + COMPRESSED_BOGUS)) as f:
+            self.assertEqual(f.read(), INPUT * 5)
+
     def test_read_from_file(self):
         with TempFile(TESTFN, COMPRESSED_XZ):
             with LZMAFile(TESTFN) as f:
@@ -719,6 +738,10 @@ class FileTestCase(unittest.TestCase):
         with LZMAFile(BytesIO(COMPRESSED_XZ)) as f:
             self.assertRaises(TypeError, f.read, None)
 
+    def test_read_bad_data(self):
+        with LZMAFile(BytesIO(COMPRESSED_BOGUS)) as f:
+            self.assertRaises(LZMAError, f.read)
+
     def test_read1(self):
         with LZMAFile(BytesIO(COMPRESSED_XZ)) as f:
             blocks = []
@@ -1237,6 +1260,8 @@ LAERTES
        Farewell.
 """
 
+COMPRESSED_BOGUS = b"this is not a valid lzma stream"
+
 COMPRESSED_XZ = (
     b"\xfd7zXZ\x00\x00\x04\xe6\xd6\xb4F\x02\x00!\x01\x16\x00\x00\x00t/\xe5\xa3"
     b"\xe0\x07\x80\x03\xdf]\x00\x05\x14\x07bX\x19\xcd\xddn\x98\x15\xe4\xb4\x9d"
